/**
 * Copyright (C) 2001-2003 France Telecom R&D
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package org.objectweb.util.monolog.wrapper.log4jMini;

import org.apache.log4j.FileAppender;
import org.apache.log4j.PatternLayout;
import org.objectweb.util.monolog.api.Handler;

import java.io.IOException;
import java.util.Hashtable;
import java.util.Enumeration;

/**
 * This class is the wrapper to the org.apache.log4j.FileAppender
 *
 * @author Sebastien Chassande-Barrioz
 */
public class FileHandler extends FileAppender implements Handler {

	protected byte type = 0;
	protected Hashtable props = null;

	/**
	 * It Builds a new MonologFileHandler. It is needed to specify an handler
	 * type.
	 * @param type 	is the handler type. The possible values are the
	 * 				HandlerFactory.FILE_HANDLER_TYPE or
	 * 				HandlerFactory.CONSOLE_HANDLER_TYPE.
	 */
	public FileHandler(byte type, String name) {
		super();
		this.type = type;
		setName(name);
		props = new Hashtable(3);
	}

	// IMPLEMENTATION OF THE Handler INTERFACE //
	//---------------------------------------------//

	public String getType() {
		return "file";
	}

	public String[] getAttributeNames() {
		String[] res = new String[props.size()];
		int i=0;
		for(Enumeration en=props.keys(); en.hasMoreElements();) {
			res[i++] = (String) en.nextElement();
		}
		return res;
	}

	public Object getAttribute(String key) {
		return props.get(key);
	}

	public Object setAttribute(String key, Object value) {
		props.put(key, value);
		if (key.equalsIgnoreCase(Handler.OUTPUT_ATTRIBUTE)) {
            if ("console".equalsIgnoreCase(type)) {
			    setOption(FILE_OPTION, (String) value);
            } else if ("file".equalsIgnoreCase(type)) {
                try {
                    fileName=(String) value;
                    setFile(fileName);
                } catch (IOException e) {
                    e.printStackTrace();
                }
            }
		}
		else if (key.equalsIgnoreCase(Handler.PATTERN_ATTRIBUTE)) {
			setLayout(
				new PatternLayout(
					PatternConverter.monolog2log4j((String) value)));
		}
		else if (key.equalsIgnoreCase(Handler.APPEND_MODE_ATTRIBUTE)) {
            setOption(APPEND_OPTION, (String) value);
			fileAppend = new Boolean( (String) value).booleanValue();
			/*try {
				setFile(fileName, fileAppend);
			}
			catch (IOException e) {
                  e.printStackTrace();
			} */
		}
		return null;
	}
}
