/**
 * Copyright (C) 2002
 */

package org.objectweb.util.monolog.wrapper.velocity;

import org.apache.velocity.runtime.RuntimeServices;
import org.apache.velocity.runtime.log.LogSystem;
import org.objectweb.util.monolog.api.BasicLevel;
import org.objectweb.util.monolog.api.Logger;

/**
 * This class permits to connect the velocity log to a monolog system.
 * To use this class you have to specify the logger that you want in the
 * constructor.
 *
 * @author Sebastien Chassande-Barrioz
 */
public class VelocityLogger implements LogSystem {

	private Logger log = null;

	public VelocityLogger(Logger l)
		throws UnsupportedOperationException {
		if (l == null) {
			throw new UnsupportedOperationException("Null logger unsupported");
		}
		this.log = l;
	}

	public void setLog(Logger l) {
		if (l == null) {
			throw new UnsupportedOperationException("Null logger unsupported");
		}
		this.log = l;
	}

	public Logger getLog() {
		return log;
	}

	// IMPLEMENTATION OF THE LogSystem INTERFACE //
	//-------------------------------------------//

	/**
	 * Nothing to do, except checking that the associated logger is not null.
	 */
	public void init(RuntimeServices services) throws Exception {
		if (log == null) {
			throw new UnsupportedOperationException("Null logger unsupported");
		}
	}

	public void logVelocityMessage(int i, String s) {
		if (i == LogSystem.WARN_ID) {
			log.log(BasicLevel.WARN, s);
		}
		else if (i == LogSystem.ERROR_ID) {
			log.log(BasicLevel.ERROR, s);
		}
		else if (i == LogSystem.INFO_ID) {
			log.log(BasicLevel.INFO, s);
		}
		else if (i == LogSystem.DEBUG_ID) {
			log.log(BasicLevel.DEBUG, s);
		}
	}
}
